@@ -67,13 +67,10 @@ computeResults <- function(obj){
   if(length(obj@input@add > 0)){
     if(grepl(":=", obj@input@add)){
       
-      ## TODO: improve code
-      pt_tmp <- parTable(m1_sem) ## TODO: what about lm()
-      pt_tmp <- pt_tmp[pt_tmp$op == ":=",]
-      npar <- nrow(pt_tmp)
-      nnewpar <- length(unlist(gregexpr(":=", obj@input@add)))
-      newnames <- pt_tmp$label[(npar-nnewpar+1):npar]
-      
+      pt_tmp <- lavaanify(model="a~1", constraints=obj@input@add) ## needs fake model
+      pt_tmp <- pt_tmp[pt_tmp$op==":=",]
+      newnames <- pt_tmp$lhs
+
       AdditionalEffects <- data.frame(est[newnames],
                                       se[newnames],
                                       tval[newnames],
@@ -181,30 +178,73 @@ computeResults <- function(obj){
 
 computeLavaanResults <- function(obj){
 
-  sem.args <- list(model=obj@syntax@model,
-                   group="cell", 
-                   missing=obj@input@missing,
-                   se=obj@input@se,
-                   fixed.x=obj@input@fixed.z,
-                   group.label=obj@input@vlevels$cell, 
-                   data=obj@input@data, 
-                   group.w.free = !obj@input@fixed.cell)
-  sem.args <- c(sem.args, obj@input@method_args)
-  m1 <- do.call("sem", sem.args)
-  
   ## lavaan.survey -- complex survey designs
   ids <- obj@input@complexsurvey$ids
   weights <- obj@input@complexsurvey$weights
   
-  if((ids != ~0) | (!is.null(weights))){
+  if((ids == ~0) & is.null(weights)){
     
-    if(!obj@input@fixed.cell){## currently only works for fixed cell sizes
-      stop("EffectLiteR error: The complex survey functionality currently only works for fixed cell sizes. Please use fixed.cell=TRUE.")
-    } 
-    survey.design <- survey::svydesign(ids=ids, weights=weights, 
-                                       data=obj@input@data)
-    m1 <- lavaan.survey::lavaan.survey(lavaan.fit=m1, 
-                                       survey.design=survey.design)    
+    sem.args <- list(model=obj@syntax@model,
+                    group="cell", 
+                    missing=obj@input@missing,
+                    se=obj@input@se,
+                    fixed.x=obj@input@fixed.z,
+                    group.label=obj@input@vlevels$cell, 
+                    data=obj@input@data, 
+                    group.w.free = !obj@input@fixed.cell)
+    sem.args <- c(sem.args, obj@input@method_args)
+    m1 <- do.call("sem", sem.args)
+    
+  }else{ # at least one lavaan.survey argument specified
+    
+    if(requireNamespace("lavaan.survey", quietly = TRUE)){ ## check if lavaan.survey is available
+      
+      if(!obj@input@fixed.cell){## currently only works for fixed cell sizes
+        stop("EffectLiteR error: The complex survey functionality currently only works for fixed cell sizes. Please use fixed.cell=TRUE.")
+      }
+      
+      sem.args <- list(model=obj@syntax@model,
+                       group="cell", 
+                       missing=obj@input@missing,
+                       se=obj@input@se,
+                       fixed.x=obj@input@fixed.z,
+                       group.label=obj@input@vlevels$cell, 
+                       data=obj@input@data, 
+                       group.w.free = !obj@input@fixed.cell)
+      sem.args <- c(sem.args, obj@input@method_args)
+      m1 <- do.call("sem", sem.args)
+      
+      survey.design <- survey::svydesign(ids=ids, weights=weights, 
+                                         data=obj@input@data)
+      m1 <- lavaan.survey::lavaan.survey(lavaan.fit=m1, 
+                                         survey.design=survey.design)
+      
+    }else{ ## lavaan.survey not installed
+      
+      warning("EffectLiteR warning: Since lavaan.survey is not installed, the lavaan:sem arguments cluster and sampling.weights are used. Only the first specified cluster variable and/or the first  specified sampling weight are used. Consider specifying these arguments directly in the call to effectLite() instead. They will be passed on to lavaan:sem.")
+      
+      ids <- all.vars(ids)[1]
+      if(is.na(ids)){ids <- NULL}
+      
+      weights <- all.vars(weights)[1]
+      if(is.na(weights)){weights <- NULL}
+      
+      
+      sem.args <- list(model=obj@syntax@model,
+                       group="cell", 
+                       missing=obj@input@missing,
+                       se=obj@input@se,
+                       fixed.x=obj@input@fixed.z,
+                       group.label=obj@input@vlevels$cell, 
+                       data=obj@input@data, 
+                       group.w.free = !obj@input@fixed.cell,
+                       cluster = ids,
+                       sampling.weights = weights)
+      sem.args <- c(sem.args, obj@input@method_args)
+      m1 <- do.call("sem", sem.args)
+      
+    }
+        
   }
   
   return(m1)
